﻿using ProjectTourismCompany.Entities.Enums;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectTourismCompany.Entities;

public class Trip
{
    public int Id { get; private set; }
    [DisplayName("Название путевки")]
    public string Title { get; private set; } = string.Empty;
    [DisplayName("Цена")]
    public int Price {  get; private set; }
    [DisplayName("Вместительность")]
    public int HumanCapacity {  get; private set; }
    [DisplayName("Дата начала")]
    public DateTime StartDate {  get; private set; }

    [DisplayName("Дата конца")]
    public DateTime EndDate { get; private set;}

    [DisplayName("Стартовый город")]
    public string StartCity {  get; private set; }=string.Empty;

    [DisplayName("Конечный город")]
    public string EndCity { get; private set;} =string.Empty;
    [DisplayName("Тип путешествия")]
    public TravelType TravelType { get; private set; }

    [DisplayName("Компании")]
    public string Company => CompanyTrip != null ?
        string.Join(", ", CompanyTrip.Select(x => $"{x.CompanyName} {x.AdditionalPrice}")) : string.Empty;

    [Browsable(false)]
    public IEnumerable<CompanyTrip> CompanyTrip
    {
        get;
        private set;
    } = [];

    public static Trip CreateOpeartion(int id, string title, int price, int humanCapacity, DateTime startDate, DateTime endDate, string startCity, string endCity, TravelType travelType, IEnumerable<CompanyTrip> companyTrip)
    {
        return new Trip
        {
            Title = title,
            Id = id,
            Price = price,
            HumanCapacity = humanCapacity,
            StartDate = startDate,
            EndDate = endDate,
            StartCity = startCity,
            EndCity = endCity,
            TravelType = travelType,
            CompanyTrip = companyTrip
        };
    }



    //public static Trip CreateOpeartion(TempCompanyTrip tempCompanyTrip,IEnumerable<CompanyTrip> companyTrip)
    //{
    //    return new Trip
    //    {
    //        Title = tempCompanyTrip.Title,
    //        Id = tempCompanyTrip.Id,
    //        Price = tempCompanyTrip.Price,
    //        HumanCapacity=tempCompanyTrip.HumanCapacity,
    //        StartDate = tempCompanyTrip.StartDate,
    //        EndDate = tempCompanyTrip.EndDate,
    //        StartCity = tempCompanyTrip.StartCity,
    //        EndCity = tempCompanyTrip.EndCity,
    //        TravelType = tempCompanyTrip.TravelType,
    //        CompanyTrip = companyTrip

    //    };
    //}


    public void SetCompanyTrip(IEnumerable<CompanyTrip> companyTrips) 
    {
        if (companyTrips != null && companyTrips.Any())
        {
            CompanyTrip = companyTrips;
        }
    }
}
