﻿using Microsoft.VisualBasic.FileIO;
using ProjectTourismCompany.Entities;
using ProjectTourismCompany.Repositories;
using ProjectTourismCompany.Repositories.Implementations;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectTourismCompany.Forms;

public partial class FormCheck : Form
{
    private readonly ICheckRepository _checkRepository;
    private int _checkId;
    public int Id
    {
        set
        {
            try
            {
                var check = _checkRepository.ReadCheckById(value);
                if (check == null)
                {
                    throw new
                    InvalidDataException(nameof(check));
                }
                dateTimePickerPurchaseDate.Value = check.PurchaseDate;
                numericUpDownPrice.Value = check.Price;
                _checkId = value;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при получении данных", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
        }
    }
    public FormCheck(ICheckRepository checkRepository, IClientRepository clientRepository, ITripRepository tripRepository)
    {
        InitializeComponent();
        _checkRepository = checkRepository ?? throw new ArgumentNullException(nameof(checkRepository));
        
        comboBoxClient.DataSource = clientRepository.ReadClients();
        comboBoxClient.DisplayMember = "Description";
        comboBoxClient.ValueMember = "Id";
        
        comboBoxTripId.DataSource = tripRepository.ReadTrips();
        comboBoxTripId.DisplayMember = "Title";
        comboBoxTripId.ValueMember = "Id";
        

    }
    private void buttonSave_Click(object sender, EventArgs e)
    {
        try
        {
            if (comboBoxTripId.SelectedIndex < 0 ||
            comboBoxClient.SelectedIndex < 0)
            {
                throw new Exception("Имеются незаполненные поля");
            }
            _checkRepository.CreateCheck(Check.CreateCheck(_checkId , dateTimePickerPurchaseDate.Value,
                (int)numericUpDownPrice.Value, (int)comboBoxTripId.SelectedValue!, 
                (int)comboBoxClient.SelectedValue!));
            Close();
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "Ошибка при сохранении",
            MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }

    private void ButtonCancel_Click(object sender, EventArgs e) =>
    Close();


}


