﻿using ProjectTourismCompany.Repositories;
using ProjectTourismCompany.Repositories.Implementations;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Unity;

namespace ProjectTourismCompany.Forms;

public partial class FormChecks : Form
{
    private readonly IUnityContainer _container;
    private readonly ICheckRepository _checkRepository;

    public FormChecks(IUnityContainer container, ICheckRepository checkRepository)
    {
        InitializeComponent();
        _container = container ??
        throw new ArgumentNullException(nameof(container));
        _checkRepository = checkRepository ??
        throw new
        ArgumentNullException(nameof(checkRepository));
    }


    private void FormChecks_Load(object sender, EventArgs e)
    {
        try
        {
            LoadList();
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "Ошибка при загрузке",
            MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }


    private void ButtonAdd_Click(object sender, EventArgs e)
    {
        try
        {
            _container.Resolve<FormCheck>().ShowDialog();
            LoadList();
        }
        catch (Exception ex)
        {
        MessageBox.Show(ex.Message, "Ошибка при добавлении",
        MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }

    private void buttonDel_Click(object sender, EventArgs e)
    {
        if (!TryGetIdentifierFromSelectedRow(out var findId))
        {
            return;
        }
        if (MessageBox.Show("Удалить запись?", "Удаление",
        MessageBoxButtons.YesNo) != DialogResult.Yes)
        {
            return;
        }
        try
        {
            _checkRepository.DeleteCheck(findId);
            LoadList();
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "Ошибка при удалении",
            MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }

    private bool TryGetIdentifierFromSelectedRow(out int id)
    {
        id = 0;
        if (dataGridViewChecks.SelectedRows.Count < 1)
        {
            MessageBox.Show("Нет выбранной записи", "Ошибка",
            MessageBoxButtons.OK, MessageBoxIcon.Error);
            return false;
        }
        id = Convert.ToInt32(dataGridViewChecks.SelectedRows[0].Cells["Id"].Value);
        return true;
    }


    private void LoadList()
    {
        dataGridViewChecks.DataSource = _checkRepository.ReadChecks();
        dataGridViewChecks.Columns["Id"].Visible = false;
        dataGridViewChecks.Columns["clientId"].Visible = false;
        dataGridViewChecks.Columns["tripId"].Visible = false;
        dataGridViewChecks.Columns["PurchaseDate"].DefaultCellStyle.Format = "dd:MM:yyyy";

    }
}

