﻿using Microsoft.VisualBasic.FileIO;
using ProjectTourismCompany.Entities.Enums;
using ProjectTourismCompany.Repositories;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectTourismCompany.Forms;

public partial class FormClient : Form
{

    private readonly IClientRepository _clientRepository;
    private int? _clientId;
    public int Id
    {
        set
        {
            try
            {
                var client = _clientRepository.ReadClientById(value);
                if (client == null)
                {
                    throw new InvalidDataException(nameof(client));
                }

                textBoxName.Text = client.Name;
                textBoxPhoneNumber.Text = client.Phone;
                comboBoxStatus.SelectedItem = client.Status;
                dateTimePickerBirthDate.Value = client.BirthDate;
                _clientId = value;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при получении данных", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
        }
    }


    public FormClient(IClientRepository clientRepository)
    {
        InitializeComponent();
        _clientRepository = clientRepository ??
        throw new
        ArgumentNullException(nameof(clientRepository));
        comboBoxStatus.DataSource =
Enum.GetValues(typeof(ClientStatus));
    }

    private void buttonSave_Click(object sender, EventArgs e)
    {
        try
        {
            if (string.IsNullOrWhiteSpace(textBoxName.Text)
            ||
            string.IsNullOrWhiteSpace(textBoxPhoneNumber.Text))
            {
                throw new Exception("Имеются незаполненные поля");
            }
            if (_clientId.HasValue)
            {
                _clientRepository.UpdateClient(CreateClient(_clientId.Value));
            }
            else
            {
                _clientRepository.CreateClient(CreateClient(0));
            }
            Close();
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "Ошибка при сохранении",
            MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }

    private void ButtonCancel_Click(object sender, EventArgs e) => Close();

    private Client CreateClient(int id)
    {
        return Client.CreateClient(id, textBoxName.Text, dateTimePickerBirthDate.Value, (ClientStatus)comboBoxStatus.SelectedItem!, textBoxPhoneNumber.Text);
    }
}
