﻿using ProjectTourismCompany.Reports;
using ProjectTourismCompany.Repositories;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Unity;

namespace ProjectTourismCompany.Forms;

public partial class FormHumanReport : Form
{
    private readonly IUnityContainer _container;
    public FormHumanReport(IUnityContainer container, IClientRepository tripRepository, ICompanyRepository companyRepository)
    {
        InitializeComponent();
        _container = container ?? throw new ArgumentNullException(nameof(container));

        comboBoxClient.DataSource = tripRepository.ReadClients();
        comboBoxClient.DisplayMember = "Name";
        comboBoxClient.ValueMember = "Id";

        comboBoxCompany.DataSource = companyRepository.ReadCompanies();
        comboBoxCompany.DisplayMember = "Name";
        comboBoxCompany.ValueMember = "Id";
    }

    private void buttonSelectFilePath_Click(object sender, EventArgs e)
    {
        var sfd = new SaveFileDialog()
        {
            Filter = "Excel Files | *.xlsx"
        };
        if (sfd.ShowDialog() != DialogResult.OK)
        {
            return;
        }
        textBoxFilePath.Text = sfd.FileName;
    }

    private void buttonMakeReport_Click(object sender, EventArgs e)
    {
        try
        {
            if (string.IsNullOrWhiteSpace(textBoxFilePath.Text))
            {
                throw new Exception("Отсутствует имя файла для отчета");
            }
            if (comboBoxCompany.SelectedIndex < 0 || comboBoxClient.SelectedIndex < 0)
            {
                throw new Exception("Не выбрана Компания или Клиент");
            }
            if (dateTimePickerStartDate.Value >= dateTimePickerEndDate.Value)
            {
                throw new Exception("Дата начала должна быть раньше даты окончания");
            }
            if (_container.Resolve<TableReport>().CreateTable(textBoxFilePath.Text, (int)comboBoxCompany.SelectedValue!,
                (int)comboBoxClient.SelectedValue!, dateTimePickerStartDate.Value, dateTimePickerEndDate.Value))
            {
                MessageBox.Show("Документ сформирован",
                "Формирование документа",
                MessageBoxButtons.OK,
                MessageBoxIcon.Information);
            }
            else
            {
                MessageBox.Show("Возникли ошибки при формировании документа.Подробности в логах",
                "Формирование документа",
                MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "Ошибка при создании очета",
            MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }
}
