using Unity.Lifetime;
using Unity;
using ProjectTourismCompany.Repositories;
using ProjectTourismCompany.Repositories.Implementations;
using ProjectTourismCompany;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;
using Serilog;
using Unity.Microsoft.Logging;

internal static class Program
{
    /// <summary>
    /// The main entry point for the application.
    /// </summary>
    [STAThread]
    static void Main()
    {

        ApplicationConfiguration.Initialize();
        Application.Run(CreateContainer().Resolve<FormTourismCompany>());
    }
    private static IUnityContainer CreateContainer()
    {
        var container = new UnityContainer();
        container.AddExtension(new LoggingExtension(CreateLoggerFactory()));

        container.RegisterType<ICheckRepository, CheckRepository>(new TransientLifetimeManager());
        container.RegisterType<IClientRepository, ClientRepository>(new TransientLifetimeManager());
        container.RegisterType<ICompanyRepository, CompanyRepository>(new TransientLifetimeManager());
        container.RegisterType<ICountryRepository, CountryRepository>(new TransientLifetimeManager());
        container.RegisterType<ITripRepository, TripRepository>();

        container.RegisterType<IConnectionString, ConnectionString>();

        return container;
    }

    private static LoggerFactory CreateLoggerFactory()
    {
        var loggerFactory = new LoggerFactory();
        loggerFactory.AddSerilog(new LoggerConfiguration()
            .ReadFrom.Configuration(new ConfigurationBuilder()
            .SetBasePath(Directory.GetCurrentDirectory())
            .AddJsonFile("appsettings.json")
            .Build())
            .CreateLogger());
        return loggerFactory;
    }
}
