﻿using Microsoft.Extensions.Logging;
using ProjectTourismCompany.Repositories;
using ProjectTourismCompany.Repositories.Implementations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection.PortableExecutable;
using System.Text;
using System.Threading.Tasks;

namespace ProjectTourismCompany.Reports;

public class ChartReport
{
    private readonly ITripRepository _tripRepository;
    private readonly ILogger<ChartReport> _logger;

    public ChartReport(ITripRepository tripRepository, ILogger<ChartReport> logger)
    {
        _tripRepository = tripRepository ?? throw new ArgumentNullException(nameof(tripRepository));
        _logger = logger ?? throw new ArgumentNullException(nameof(logger));
    }
    public bool CreateChart(string filePath, int companyId, DateTime dateTime)
    {
        try
        {
            new PdfBuilder(filePath)
                .AddHeader("Туры компаний")
                .AddPieChart("Виды туров", GetData(companyId, dateTime))
                .Build();
            return true;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при формировании документа");
            return false;
        }
    }
    private List<(string Caption, double Value)> GetData(int companyId, DateTime dateTime)
    {
        return _tripRepository
            .ReadTrips()
            .Where(x => x.StartDate.Date >=  dateTime.Date)
            .GroupBy(x => x.StartCity, (key, group) => new
            {
                Id = key,
                Count = group.Sum(y => y.CompanyTrip.Count(z => z.CompanyId == companyId))
            })
            .Select(x => (x.Id.ToString(), (double)x.Count))
            .ToList();
    }
}