﻿using Microsoft.Extensions.Logging;
using ProjectTourismCompany.Repositories;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectTourismCompany.Reports;

public class DocReport
{
    private readonly ICompanyRepository _companyRepository;
    private readonly IClientRepository _clientRepository;
    private readonly ICountryRepository _countryRepository;
    private readonly ILogger<DocReport> _logger;    

    public DocReport(ICompanyRepository companyRepository, IClientRepository clientRepository, ICountryRepository countryRepository, ILogger<DocReport> logger) 
    {
        _clientRepository = clientRepository ?? throw new ArgumentNullException(nameof(clientRepository));
        _companyRepository = companyRepository ?? throw new ArgumentNullException(nameof(companyRepository)); 
        _countryRepository = countryRepository ?? throw new ArgumentNullException(nameof(countryRepository));
        _logger = logger ?? throw new ArgumentNullException(nameof(logger));

    }


        
    public bool CreateDoc(string filePath, bool includeCompany, bool includeClient, bool includeCountry)
    {
        try
        {
            var builder = new WordBuilder(filePath).AddHeader("Документ со справочниками");
            if (includeCompany)
            {
                builder.AddParagraph("Компании").AddTable([2400, 1200], GetCompanies());
            }
            if (includeClient)
            {
                builder.AddParagraph("Клиенты").AddTable([2400, 1200, 2400, 2400], GetClients());
            }
            if (includeCountry)
            {
                builder.AddParagraph("Государства").AddTable([2400], GetCountries());
            }
            builder.Build();
            return true;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при формировании документа");
            return false;
        }
    }

    private List<string[]> GetCompanies()
    {
        return [
            ["Название компании", "id государства"],
            .. _companyRepository
                .ReadCompanies()
                .Select(x => new string[] { x.Name, x.CountryId.ToString() }),
        ];
    }

    private List<string[]> GetClients()
    {
        return [
            ["ФИО", "Дата рождения", "Статус", "Номер телефона"],
            .. _clientRepository
                .ReadClients()
                .Select(x => new string[] { x.Name, x.BirthDate.ToString(), x.Status.ToString(), x.Phone }),
        ];
    }

    private List<string[]> GetCountries()
    {
        return [
            ["Название государства"],
            .. _countryRepository
                .ReadCountries()
                .Select(x => new string[] { x.CountryName }),
        ];
    }
}