﻿using Microsoft.Extensions.Logging;
using ProjectTourismCompany.Repositories;
using ProjectTourismCompany.Repositories.Implementations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectTourismCompany.Reports;

public class TableReport
{
    private readonly ICheckRepository _checkRepository;
    private readonly ITripRepository _tripRepository;
    private readonly ILogger<TableReport> _logger;
    internal static readonly string[] item = ["Клиент", "Дата начала", "Цена тура", "Возвращенный залог"];
    public TableReport(ICheckRepository checkRepository, ITripRepository tripRepository, ILogger<TableReport> logger)
    {
        _checkRepository = checkRepository ?? throw new ArgumentNullException(nameof(checkRepository));
        _tripRepository = tripRepository ?? throw new ArgumentNullException(nameof(tripRepository));
        _logger = logger ?? throw new ArgumentNullException(nameof(logger));
    }


    public bool CreateTable(string filePath, int companyId, int clientId, DateTime startDate, DateTime endDate)
    {
        try
        {
            new ExcelBuilder(filePath)
                .AddHeader("Сводка по движению денег", 0, 4)
                .AddParagraph("за период", 0)
                .AddTable([15, 10, 15, 15], GetData(clientId, companyId, startDate, endDate))
                .Build();
            return true;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при формировании документа");
            return false;
        }
    }

    private List<string[]> GetData(int clientId, int companyId, DateTime startDate, DateTime endDate)
    {
        var data = _checkRepository
            .ReadChecks()
            .Where(x => x.ClientId == clientId && startDate < x.PurchaseDate && endDate > x.PurchaseDate)
            .Select(x => new {human = (int?)x.ClientId, Date = x.PurchaseDate, CountIn = (int?)x.Price, CountOut = (int?)null })
            .Union(
                _tripRepository
                .ReadTrips()
                //.Where(x => x.CompanyTrip.Any(y => y.CompanyId == companyId))
                .Select(x => new { human = (int?)null, Date = x.EndDate, CountIn = (int?)null, CountOut = (int?)100  })) //.CompanyTrip.FirstOrDefault(y => y.CompanyId == companyId).AdditionalPrice
            .OrderBy(x => x.Date);
        return
            new List<string[]>() { item }
            .Union(data.Select(x => new string[] {x.human.ToString() ?? string.Empty, x.Date.ToString(), x.CountIn?.ToString() ?? string.Empty, x.CountOut?.ToString() ?? string.Empty }))
            .Union(
                [["Всего", "", data.Sum(x => x.CountIn ?? 0).ToString(), data.Sum(x => x.CountOut ?? 0).ToString()]]
            )
            .ToList();
    }
}